@@ -43,48 +43,56 @@ namespace aria2 {
 class AbstractDiskWriter : public DiskWriter {
 private:
   std::string filename_;
+
+#ifdef __MINGW32__
+  HANDLE fd_;
+  // The handle for memory mapped file. mmap equivalent in Windows.
+  HANDLE mapView_;
+#else // !__MINGW32__
   int fd_;
+#endif // !__MINGW32__
 
   bool readOnly_;
 
-  bool directIOAllowed_;
+  bool enableMmap_;
+  unsigned char* mapaddr_;
+  int64_t maplen_;
 
-  ssize_t writeDataInternal(const unsigned char* data, size_t len);
-  ssize_t readDataInternal(unsigned char* data, size_t len);
+  ssize_t writeDataInternal(const unsigned char* data, size_t len,
+                            int64_t offset);
+  ssize_t readDataInternal(unsigned char* data, size_t len, int64_t offset);
 
-  void seek(off_t offset);
+  void seek(int64_t offset);
+
+  void ensureMmapWrite(size_t len, int64_t offset);
 protected:
   void createFile(int addFlags = 0);
 public:
   AbstractDiskWriter(const std::string& filename);
   virtual ~AbstractDiskWriter();
 
-  virtual void openFile(uint64_t totalLength = 0);
+  virtual void openFile(int64_t totalLength = 0);
 
   virtual void closeFile();
 
-  virtual void openExistingFile(uint64_t totalLength = 0);
+  virtual void openExistingFile(int64_t totalLength = 0);
 
-  virtual void writeData(const unsigned char* data, size_t len, off_t offset);
+  virtual void writeData(const unsigned char* data, size_t len, int64_t offset);
 
-  virtual ssize_t readData(unsigned char* data, size_t len, off_t offset);
+  virtual ssize_t readData(unsigned char* data, size_t len, int64_t offset);
 
-  virtual void truncate(uint64_t length);
+  virtual void truncate(int64_t length);
 
   // File must be opened before calling this function.
-  virtual void allocate(off_t offset, uint64_t length);
+  virtual void allocate(int64_t offset, int64_t length, bool sparse);
 
-  virtual uint64_t size();
-  
-  virtual void enableDirectIO();
-
-  virtual void disableDirectIO();
-
-  virtual void allowDirectIO() { directIOAllowed_ = true; }
+  virtual int64_t size();
 
   virtual void enableReadOnly();
 
   virtual void disableReadOnly();
+
+  virtual void enableMmap();
 };
 
 } // namespace aria2
