@@ -35,12 +35,17 @@
 #include "BtNotInterestedMessage.h"
 #include "Peer.h"
 #include "PeerStorage.h"
+#include "SocketBuffer.h"
 
 namespace aria2 {
 
-const std::string BtNotInterestedMessage::NAME("not interested");
+const char BtNotInterestedMessage::NAME[] = "not interested";
 
-SharedHandle<BtNotInterestedMessage> BtNotInterestedMessage::create
+BtNotInterestedMessage::BtNotInterestedMessage():ZeroBtMessage(ID, NAME) {}
+
+BtNotInterestedMessage::~BtNotInterestedMessage() {}
+
+BtNotInterestedMessage* BtNotInterestedMessage::create
 (const unsigned char* data, size_t dataLength)
 {
   return ZeroBtMessage::create<BtNotInterestedMessage>(data, dataLength);
@@ -48,28 +53,43 @@ SharedHandle<BtNotInterestedMessage> BtNotInterestedMessage::create
 
 void BtNotInterestedMessage::doReceivedAction()
 {
-  if(_metadataGetMode) {
+  if(isMetadataGetMode()) {
     return;
   }
-  peer->peerInterested(false);
-  if(!peer->amChoking()) {
-    _peerStorage->executeChoke();
+  getPeer()->peerInterested(false);
+  if(!getPeer()->amChoking()) {
+    peerStorage_->executeChoke();
   }
 }
 
 bool BtNotInterestedMessage::sendPredicate() const
 {
-  return peer->amInterested();
+  return getPeer()->amInterested();
 }
 
-void BtNotInterestedMessage::onSendComplete() {
-  peer->amInterested(false);
+namespace {
+struct ThisProgressUpdate : public ProgressUpdate {
+  ThisProgressUpdate(const SharedHandle<Peer>& peer)
+    : peer(peer) {}
+  virtual void update(size_t length, bool complete)
+  {
+    if(complete) {
+      peer->amInterested(false);
+    }
+  }
+  SharedHandle<Peer> peer;
+};
+} // namespace
+
+ProgressUpdate* BtNotInterestedMessage::getProgressUpdate()
+{
+  return new ThisProgressUpdate(getPeer());
 }
 
 void BtNotInterestedMessage::setPeerStorage
 (const SharedHandle<PeerStorage>& peerStorage)
 {
-  _peerStorage = peerStorage;
+  peerStorage_ = peerStorage;
 }
 
 } // namespace aria2
