@@ -38,6 +38,7 @@
 #include "StatCalc.h"
 
 #include <string>
+#include <functional>
 
 #include "TimerA2.h"
 
@@ -63,6 +64,8 @@ private:
 
   SharedHandle<SizeFormatter> sizeFormatter_;
   bool readoutVisibility_;
+  bool truncate_;
+  bool isTTY_;
 public:
   ConsoleStatCalc(time_t summaryInterval, bool humanReadable = true);
 
@@ -74,9 +77,12 @@ public:
   {
     readoutVisibility_ = visibility;
   }
-};
 
-typedef SharedHandle<ConsoleStatCalc> ConsoleStatCalcHandle;
+  void setTruncate(bool truncate)
+  {
+    truncate_ = truncate;
+  }
+};
 
 } // namespace aria2
 
