@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DISK_ADAPTOR_H_
-#define _D_DISK_ADAPTOR_H_
+#ifndef D_DISK_ADAPTOR_H
+#define D_DISK_ADAPTOR_H
 
 #include "BinaryStream.h"
 
@@ -41,28 +41,24 @@
 #include <vector>
 
 #include "TimeA2.h"
+#include "SharedHandle.h"
 
 namespace aria2 {
 
 class FileEntry;
-class Logger;
 class FileAllocationIterator;
+class WrDiskCacheEntry;
 
 class DiskAdaptor:public BinaryStream {
-private:
-  std::vector<SharedHandle<FileEntry> > fileEntries_;
-#ifdef HAVE_POSIX_FALLOCATE
-  bool fallocate_;
-#endif // HAVE_POSIX_FALLOCATE
-  Logger* logger_;
-protected:
-  Logger* getLogger() const
-  {
-    return logger_;
-  }
 public:
+  enum FileAllocationMethod {
+    FILE_ALLOC_ADAPTIVE,
+    FILE_ALLOC_FALLOC,
+    FILE_ALLOC_TRUNC
+  };
+
   DiskAdaptor();
-  virtual ~DiskAdaptor() {}
+  virtual ~DiskAdaptor();
 
   virtual void openFile() = 0;
 
@@ -74,7 +70,7 @@ public:
 
   virtual bool fileExists() = 0;
 
-  virtual uint64_t size() = 0;
+  virtual int64_t size() = 0;
 
   template<typename InputIterator>
   void setFileEntries(InputIterator first, InputIterator last)
@@ -89,16 +85,16 @@ public:
 
   virtual SharedHandle<FileAllocationIterator> fileAllocationIterator() = 0;
 
-  virtual void enableDirectIO() {}
-
-  virtual void disableDirectIO() {}
-
   virtual void enableReadOnly() {}
 
   virtual void disableReadOnly() {}
 
   virtual bool isReadOnlyEnabled() const { return false; }
 
+  // Enables mmap feature. Some derived classes may require that files
+  // have been opened before this method call.
+  virtual void enableMmap() {}
+
   // Assumed each file length is stored in fileEntries or DiskAdaptor knows it.
   // If each actual file's length is larger than that, truncate file to that
   // length.
@@ -110,26 +106,25 @@ public:
   // successfully changed.
   virtual size_t utime(const Time& actime, const Time& modtime) = 0;
 
-#ifdef HAVE_POSIX_FALLOCATE
-  void enableFallocate()
+  // Writes cached data to the underlying disk.
+  virtual void writeCache(const WrDiskCacheEntry* entry) = 0;
+
+  void setFileAllocationMethod(FileAllocationMethod method)
   {
-    fallocate_ = true;
+    fileAllocationMethod_ = method;
   }
 
-  void disableFallocate()
+  int getFileAllocationMethod() const
   {
-    fallocate_ = false;
+    return fileAllocationMethod_;
   }
 
-  bool doesFallocate() const
-  {
-    return fallocate_;
-  }
-#endif // HAVE_POSIX_FALLOCATE
+private:
+  std::vector<SharedHandle<FileEntry> > fileEntries_;
+
+  FileAllocationMethod fileAllocationMethod_;
 };
 
-typedef SharedHandle<DiskAdaptor> DiskAdaptorHandle;
-
 } // namespace aria2
 
-#endif // _D_DISK_ADAPTOR_H_
+#endif // D_DISK_ADAPTOR_H
