@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DOWNLOAD_CONTEXT_H_
-#define _D_DOWNLOAD_CONTEXT_H_
+#ifndef D_DOWNLOAD_CONTEXT_H
+#define D_DOWNLOAD_CONTEXT_H
 
 #include "common.h"
 
@@ -42,34 +42,33 @@
 #include <vector>
 
 #include "SharedHandle.h"
-#include "Signature.h"
 #include "TimerA2.h"
 #include "A2STR.h"
 #include "ValueBase.h"
-#include "IntSequence.h"
-#include "FileEntry.h"
-#include "TorrentAttribute.h"
+#include "SegList.h"
+#include "ContextAttribute.h"
+#include "NetStat.h"
 
 namespace aria2 {
 
 class RequestGroup;
+class Signature;
+class FileEntry;
 
 class DownloadContext
 {
 private:
   std::vector<SharedHandle<FileEntry> > fileEntries_;
 
-  std::string dir_;
-
   std::vector<std::string> pieceHashes_;
 
-  size_t pieceLength_;
+  int32_t pieceLength_;
 
-  std::string pieceHashAlgo_;
+  std::string pieceHashType_;
 
-  std::string checksum_;
+  std::string digest_;
 
-  std::string checksumHashAlgo_;
+  std::string hashType_;
 
   bool checksumVerified_;
 
@@ -78,44 +77,45 @@ private:
   bool knowsTotalLength_;
 
   RequestGroup* ownerRequestGroup_;
-  
-  std::map<std::string, SharedHandle<ContextAttribute> > attrs_;
 
-  Timer downloadStartTime_;
+  std::vector<SharedHandle<ContextAttribute> > attrs_;
+
+  NetStat netStat_;
 
   Timer downloadStopTime_;
 
   SharedHandle<Signature> signature_;
+  // This member variable is required to avoid to use parse Metalink
+  // (including both Metalink XML and Metalink/HTTP) twice.
+  bool acceptMetalink_;
 public:
   DownloadContext();
 
   // Convenient constructor that creates single file download.  path
   // should be escaped with util::escapePath(...).
-  DownloadContext(size_t pieceLength,
-                  uint64_t totalLength,
+  DownloadContext(int32_t pieceLength,
+                  int64_t totalLength,
                   const std::string& path = A2STR::NIL);
 
-  const std::string& getPieceHash(size_t index) const
-  {
-    if(index < pieceHashes_.size()) {
-      return pieceHashes_[index];
-    } else {
-      return A2STR::NIL;
-    }
-  }
-  
+  ~DownloadContext();
+
+  const std::string& getPieceHash(size_t index) const;
+
   const std::vector<std::string>& getPieceHashes() const
   {
     return pieceHashes_;
   }
 
   template<typename InputIterator>
-  void setPieceHashes(InputIterator first, InputIterator last)
+  void setPieceHashes
+  (const std::string& hashType,
+   InputIterator first, InputIterator last)
   {
+    pieceHashType_ = hashType;
     pieceHashes_.assign(first, last);
   }
 
-  uint64_t getTotalLength() const;
+  int64_t getTotalLength() const;
 
   bool knowsTotalLength() const { return knowsTotalLength_; }
 
@@ -134,38 +134,32 @@ public:
     return fileEntries_[0];
   }
 
+  // This function returns first FileEntry whose isRequested() returns
+  // true.  If there is no such FileEntry, returns
+  // SharedHandle<FileEntry>().
+  SharedHandle<FileEntry> getFirstRequestedFileEntry() const;
+
+  size_t countRequestedFileEntry() const;
+
   template<typename InputIterator>
   void setFileEntries(InputIterator first, InputIterator last)
   {
     fileEntries_.assign(first, last);
   }
 
-  size_t getPieceLength() const { return pieceLength_; }
+  int32_t getPieceLength() const { return pieceLength_; }
 
-  void setPieceLength(size_t length) { pieceLength_ = length; }
+  void setPieceLength(int32_t length) { pieceLength_ = length; }
 
   size_t getNumPieces() const;
 
-  const std::string& getPieceHashAlgo() const { return pieceHashAlgo_; }
+  const std::string& getPieceHashType() const { return pieceHashType_; }
 
-  void setPieceHashAlgo(const std::string& algo)
-  {
-    pieceHashAlgo_ = algo;
-  }
+  const std::string& getDigest() const { return digest_; }
 
-  const std::string& getChecksum() const { return checksum_; }
+  const std::string& getHashType() const { return hashType_; }
 
-  void setChecksum(const std::string& checksum)
-  {
-    checksum_ = checksum;
-  }
-
-  const std::string& getChecksumHashAlgo() const { return checksumHashAlgo_; }
-
-  void setChecksumHashAlgo(const std::string& algo)
-  {
-    checksumHashAlgo_ = algo;
-  }
+  void setDigest(const std::string& hashType, const std::string& digest);
 
   // The representative path name for this context. It is used as a
   // part of .aria2 control file. If basePath_ is set, returns
@@ -173,18 +167,11 @@ public:
   // returned.
   const std::string& getBasePath() const;
 
-  void setBasePath(const std::string& basePath) { basePath_ = basePath; }
-
-  const std::string& getDir() const { return dir_; }
-
-  void setDir(const std::string& dir) { dir_ = dir; }
+  void setBasePath(const std::string& basePath);
 
   const SharedHandle<Signature>& getSignature() const { return signature_; }
 
-  void setSignature(const SharedHandle<Signature>& signature)
-  {
-    signature_ = signature;
-  }
+  void setSignature(const SharedHandle<Signature>& signature);
 
   RequestGroup* getOwnerRequestGroup() { return ownerRequestGroup_; }
 
@@ -193,13 +180,12 @@ public:
     ownerRequestGroup_ = owner;
   }
 
-  void setFileFilter(IntSequence seq);
+  // sgl must be normalized before the call.
+  void setFileFilter(SegList<int>& sgl);
 
   // Sets file path for specified index. index starts from 1. The
-  // index is the same used in setFileFilter().  Please note that path
-  // is not the actual file path. The actual file path is
-  // getDir()+"/"+path. path is not escaped by util::escapePath() in
-  // this function.
+  // index is the same used in setFileFilter(). path is not escaped by
+  // util::escapePath() in this function.
   void setFilePathWithIndex(size_t index, const std::string& path);
 
   // Returns true if hash check(whole file hash, not piece hash) is
@@ -218,11 +204,11 @@ public:
   }
 
   void setAttribute
-  (const std::string& key, const SharedHandle<ContextAttribute>& value);
+  (ContextAttributeType key, const SharedHandle<ContextAttribute>& value);
 
-  const SharedHandle<ContextAttribute>& getAttribute(const std::string& key);
+  const SharedHandle<ContextAttribute>& getAttribute(ContextAttributeType key);
 
-  bool hasAttribute(const std::string& key) const;
+  bool hasAttribute(ContextAttributeType key) const;
 
   void resetDownloadStartTime();
 
@@ -234,14 +220,36 @@ public:
   }
 
   int64_t calculateSessionTime() const;
-  
+
   // Returns FileEntry at given offset. SharedHandle<FileEntry>() is
   // returned if no such FileEntry is found.
-  SharedHandle<FileEntry> findFileEntryByOffset(off_t offset) const;
+  SharedHandle<FileEntry> findFileEntryByOffset(int64_t offset) const;
 
   void releaseRuntimeResource();
+
+  void setAcceptMetalink(bool f)
+  {
+    acceptMetalink_ = f;
+  }
+  bool getAcceptMetalink() const
+  {
+    return acceptMetalink_;
+  }
+
+  NetStat& getNetStat()
+  {
+    return netStat_;
+  }
+
+  // This method also updates global download length held by
+  // RequestGroupMan via getOwnerRequestGroup().
+  void updateDownloadLength(size_t bytes);
+
+  // This method also updates global upload length held by
+  // RequestGroupMan via getOwnerRequestGroup().
+  void updateUploadLength(size_t bytes);
 };
 
 } // namespace aria2
 
-#endif // _D_DOWNLOAD_CONTEXT_H_
+#endif // D_DOWNLOAD_CONTEXT_H
