@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DOWNLOAD_ENGINE_H_
-#define _D_DOWNLOAD_ENGINE_H_
+#ifndef D_DOWNLOAD_ENGINE_H
+#define D_DOWNLOAD_ENGINE_H
 
 #include "common.h"
 
@@ -46,17 +46,16 @@
 #include "a2netcompat.h"
 #include "TimerA2.h"
 #include "a2io.h"
-#ifdef ENABLE_ASYNC_DNS
-# include "AsyncNameResolver.h"
-#endif // ENABLE_ASYNC_DNS
 #include "CUIDCounter.h"
 #include "FileAllocationMan.h"
 #include "CheckIntegrityMan.h"
 #include "DNSCache.h"
+#ifdef ENABLE_ASYNC_DNS
+# include "AsyncNameResolver.h"
+#endif // ENABLE_ASYNC_DNS
 
 namespace aria2 {
 
-class Logger;
 class Option;
 class RequestGroupMan;
 class StatCalc;
@@ -78,8 +77,6 @@ private:
 
   SharedHandle<EventPoll> eventPoll_;
 
-  Logger* logger_;
-  
   SharedHandle<StatCalc> statCalc_;
 
   bool haltRequested_;
@@ -87,15 +84,15 @@ private:
   class SocketPoolEntry {
   private:
     SharedHandle<SocketCore> socket_;
-
-    std::map<std::string, std::string> options_;
+    // protocol specific option string
+    std::string options_;
 
     time_t timeout_;
 
     Timer registeredTime_;
   public:
     SocketPoolEntry(const SharedHandle<SocketCore>& socket,
-                    const std::map<std::string, std::string>& option,
+                    const std::string& option,
                     time_t timeout);
 
     SocketPoolEntry(const SharedHandle<SocketCore>& socket,
@@ -110,7 +107,7 @@ private:
       return socket_;
     }
 
-    const std::map<std::string, std::string>& getOptions() const
+    const std::string& getOptions() const
     {
       return options_;
     }
@@ -118,12 +115,12 @@ private:
 
   // key = IP address:port, value = SocketPoolEntry
   std::multimap<std::string, SocketPoolEntry> socketPool_;
- 
+
   Timer lastSocketPoolScan_;
 
   bool noWait_;
 
-  static const int64_t DEFAULT_REFRESH_INTERVAL = 900;
+  static const int64_t DEFAULT_REFRESH_INTERVAL = 1000;
 
   // Milliseconds
   int64_t refreshInterval_;
@@ -138,6 +135,10 @@ private:
 
   CUIDCounter cuidCounter_;
 
+#ifdef HAVE_ARES_ADDR_NODE
+  ares_addr_node* asyncDNSServers_;
+#endif // HAVE_ARES_ADDR_NODE
+
   SharedHandle<DNSCache> dnsCache_;
 
   SharedHandle<AuthConfigFactory> authConfigFactory_;
@@ -150,7 +151,7 @@ private:
   void onEndOfRun();
 
   void afterEachIteration();
-  
+
   void poolSocket(const std::string& key, const SocketPoolEntry& entry);
 
   std::multimap<std::string, SocketPoolEntry>::iterator
@@ -161,7 +162,7 @@ private:
   SharedHandle<FileAllocationMan> fileAllocationMan_;
   SharedHandle<CheckIntegrityMan> checkIntegrityMan_;
   Option* option_;
-public:  
+public:
   DownloadEngine(const SharedHandle<EventPoll>& eventPoll);
 
   ~DownloadEngine();
@@ -187,45 +188,30 @@ public:
                                Command* command);
 #endif // ENABLE_ASYNC_DNS
 
-  void addCommand(const std::vector<Command*>& commands)
-  {
-    commands_.insert(commands_.end(), commands.begin(), commands.end());
-  }
+  void addCommand(const std::vector<Command*>& commands);
 
-  void addCommand(Command* command)
-  {
-    commands_.push_back(command);
-  }
+  void addCommand(Command* command);
 
   const SharedHandle<RequestGroupMan>& getRequestGroupMan() const
   {
     return requestGroupMan_;
   }
 
-  void setRequestGroupMan(const SharedHandle<RequestGroupMan>& rgman)
-  {
-    requestGroupMan_ = rgman;
-  }
+  void setRequestGroupMan(const SharedHandle<RequestGroupMan>& rgman);
 
   const SharedHandle<FileAllocationMan>& getFileAllocationMan() const
   {
     return fileAllocationMan_;
   }
 
-  void setFileAllocationMan(const SharedHandle<FileAllocationMan>& faman)
-  {
-    fileAllocationMan_ = faman;
-  }
+  void setFileAllocationMan(const SharedHandle<FileAllocationMan>& faman);
 
   const SharedHandle<CheckIntegrityMan>& getCheckIntegrityMan() const
   {
     return checkIntegrityMan_;
   }
 
-  void setCheckIntegrityMan(const SharedHandle<CheckIntegrityMan>& ciman)
-  {
-    checkIntegrityMan_ = ciman;
-  }
+  void setCheckIntegrityMan(const SharedHandle<CheckIntegrityMan>& ciman);
 
   Option* getOption() const
   {
@@ -256,16 +242,16 @@ public:
                   const std::string& username,
                   const std::string& proxyhost, uint16_t proxyport,
                   const SharedHandle<SocketCore>& sock,
-                  const std::map<std::string, std::string>& options,
+                  const std::string& options,
                   time_t timeout = 15);
 
   void poolSocket(const SharedHandle<Request>& request,
                   const std::string& username,
                   const SharedHandle<Request>& proxyRequest,
                   const SharedHandle<SocketCore>& socket,
-                  const std::map<std::string, std::string>& options,
+                  const std::string& options,
                   time_t timeout = 15);
-    
+
   void poolSocket(const std::string& ipaddr, uint16_t port,
                   const std::string& proxyhost, uint16_t proxyport,
                   const SharedHandle<SocketCore>& sock,
@@ -282,7 +268,7 @@ public:
    const std::string& proxyhost, uint16_t proxyport);
 
   SharedHandle<SocketCore> popPooledSocket
-  (std::map<std::string, std::string>& options,
+  (std::string& options,
    const std::string& ipaddr,
    uint16_t port,
    const std::string& username,
@@ -294,7 +280,7 @@ public:
 
   SharedHandle<SocketCore>
   popPooledSocket
-  (std::map<std::string, std::string>& options,
+  (std::string& options,
    const std::vector<std::string>& ipaddrs,
    uint16_t port,
    const std::string& username);
@@ -344,11 +330,18 @@ public:
   {
     return sessionId_;
   }
-};
 
-typedef SharedHandle<DownloadEngine> DownloadEngineHandle;
+#ifdef HAVE_ARES_ADDR_NODE
+  void setAsyncDNSServers(ares_addr_node* asyncDNSServers);
+
+  ares_addr_node* getAsyncDNSServers() const
+  {
+    return asyncDNSServers_;
+  }
+#endif // HAVE_ARES_ADDR_NODE
+};
 
 } // namespace aria2
 
-#endif // _D_DOWNLOAD_ENGINE_H_
+#endif // D_DOWNLOAD_ENGINE_H
 
