@@ -70,10 +70,10 @@ bool FileAllocationCommand::executeInternal()
   if(fileAllocationEntry_->finished()) {
     A2_LOG_DEBUG
       (fmt(MSG_ALLOCATION_COMPLETED,
-           static_cast<long int>(timer_.difference(global::wallclock)),
-           util::itos(getRequestGroup()->getTotalLength(), true).c_str()));
+           static_cast<long int>(timer_.difference(global::wallclock())),
+           getRequestGroup()->getTotalLength()));
     getDownloadEngine()->getFileAllocationMan()->dropPickedEntry();
-    
+
     std::vector<Command*>* commands = new std::vector<Command*>();
     auto_delete_container<std::vector<Command*> > commandsDel(commands);
     fileAllocationEntry_->prepareForNextAction(*commands, getDownloadEngine());
