@@ -34,6 +34,7 @@
 /* copyright --> */
 #include "Logger.h"
 
+#include <unistd.h>
 #include <cstring>
 #include <cstdio>
 
@@ -48,21 +49,17 @@
 
 namespace aria2 {
 
-namespace {
-static const std::string DEBUG_LABEL("DEBUG");
-
-static const std::string INFO_LABEL("INFO");
-
-static const std::string NOTICE_LABEL("NOTICE");
-
-static const std::string WARN_LABEL("WARN");
-
-static const std::string ERROR_LABEL("ERROR");
-} // namespace
-
 Logger::Logger()
   : logLevel_(Logger::A2_DEBUG),
-    stdoutField_(0)
+    consoleLogLevel_(Logger::A2_NOTICE),
+    consoleOutput_(true),
+#ifdef __MINGW32__
+    // Windows DOS prompt does not handle ANSI color code, so make
+    // this false.
+    useColor_(false)
+#else // !__MINGW32__
+    useColor_(isatty(STDOUT_FILENO) == 1)
+#endif // !__MINGW32__
 {}
 
 Logger::~Logger()
@@ -75,7 +72,7 @@ void Logger::openFile(const std::string& filename)
   if(filename == DEV_STDOUT) {
     fpp_ = global::cout();
   } else {
-    fpp_.reset(new BufferedFile(filename, BufferedFile::APPEND));
+    fpp_.reset(new BufferedFile(filename.c_str(), BufferedFile::APPEND));
     if(!*static_cast<BufferedFile*>(fpp_.get())) {
       throw DL_ABORT_EX(fmt(EX_FILE_OPEN, filename.c_str(), "n/a"));
     }
@@ -89,36 +86,42 @@ void Logger::closeFile()
   }
 }
 
-void Logger::setStdoutLogLevel(Logger::LEVEL level, bool enabled)
+void Logger::setConsoleOutput(bool enabled)
 {
-  if(enabled) {
-    stdoutField_ |= level;
-  } else {
-    stdoutField_ &= ~level;
-  }
+  consoleOutput_ = enabled;
+}
+
+bool Logger::fileLogEnabled(LEVEL level)
+{
+  return level >= logLevel_ && fpp_;
+}
+
+bool Logger::consoleLogEnabled(LEVEL level)
+{
+  return consoleOutput_ && level >= consoleLogLevel_;
 }
 
 bool Logger::levelEnabled(LEVEL level)
 {
-  return (level >= logLevel_ && fpp_) || stdoutField_&level;
+  return fileLogEnabled(level) || consoleLogEnabled(level);
 }
 
 namespace {
-const std::string& levelToString(Logger::LEVEL level)
+const char* levelToString(Logger::LEVEL level)
 {
   switch(level) {
   case Logger::A2_DEBUG:
-    return DEBUG_LABEL;
+    return "DEBUG";
   case Logger::A2_INFO:
-    return INFO_LABEL;
+    return "INFO";
   case Logger::A2_NOTICE:
-    return NOTICE_LABEL;
+    return "NOTICE";
   case Logger::A2_WARN:
-    return WARN_LABEL;
+    return "WARN";
   case Logger::A2_ERROR:
-    return ERROR_LABEL;
+    return "ERROR";
   default:
-    return A2STR::NIL;
+    return "";
   }
 }
 } // namespace
@@ -138,19 +141,48 @@ void writeHeader
   size_t dateLength =
     strftime(datestr, sizeof(datestr), "%Y-%m-%d %H:%M:%S", &tm);
   assert(dateLength <= (size_t)20);
-  fp.printf("%s.%06ld %s - ", datestr, tv.tv_usec,
-            levelToString(level).c_str());
-  if(sourceFile) {
-    fp.printf("[%s:%d]", sourceFile, lineNum);
+  fp.printf("%s.%06ld [%s] [%s:%d] ", datestr, tv.tv_usec, levelToString(level),
+            sourceFile, lineNum);
+}
+} // namespace
+
+namespace {
+const char* levelColor(Logger::LEVEL level)
+{
+  switch(level) {
+  case Logger::A2_DEBUG:
+    return "\033[1;37m";
+  case Logger::A2_INFO:
+    return "\033[1;36m";
+  case Logger::A2_NOTICE:
+    return "\033[1;32m";
+  case Logger::A2_WARN:
+    return "\033[1;33m";
+  case Logger::A2_ERROR:
+    return "\033[1;31m";
+  default:
+    return "";
   }
 }
 } // namespace
 
 namespace {
 template<typename Output>
-void writeStackTrace(Output& fp, const std::string& stackTrace)
+void writeHeaderConsole(Output& fp, Logger::LEVEL level, bool useColor)
 {
-  fp.write(stackTrace.c_str());
+  if(useColor) {
+    fp.printf("[%s%s\033[0m] ", levelColor(level), levelToString(level));
+  } else {
+    fp.printf("[%s] ", levelToString(level));
+  }
+}
+} // namespace
+
+namespace {
+template<typename Output>
+void writeStackTrace(Output& fp, const char* stackTrace)
+{
+  fp.write(stackTrace);
 }
 } // namespace
 
@@ -159,19 +191,17 @@ void Logger::writeLog
  const char* sourceFile,
  int lineNum,
  const char* msg,
- const std::string& trace,
- bool toStream,
- bool toConsole)
+ const char* trace)
 {
-  if(toStream) {
+  if(fileLogEnabled(level)) {
     writeHeader(*fpp_, level, sourceFile, lineNum);
     fpp_->printf("%s\n", msg);
     writeStackTrace(*fpp_, trace);
     fpp_->flush();
   }
-  if(toConsole) {
+  if(consoleLogEnabled(level)) {
     global::cout()->printf("\n");
-    writeHeader(*global::cout(), level, 0, 0);
+    writeHeaderConsole(*global::cout(), level, useColor_);
     global::cout()->printf("%s\n", msg);
     writeStackTrace(*global::cout(), trace);
     global::cout()->flush();
@@ -184,9 +214,7 @@ void Logger::log
  int lineNum,
  const char* msg)
 {
-  writeLog(level, sourceFile, lineNum, msg, A2STR::NIL,
-           level >= logLevel_ && fpp_,
-           stdoutField_&level);
+  writeLog(level, sourceFile, lineNum, msg, "");
 }
 
 void Logger::log
@@ -205,9 +233,7 @@ void Logger::log
  const char* msg,
  const Exception& ex)
 {
-  writeLog(level, sourceFile, lineNum, msg, ex.stackTrace(),
-           level >= logLevel_ && fpp_,
-           stdoutField_&level);
+  writeLog(level, sourceFile, lineNum, msg, ex.stackTrace().c_str());
 }
 
 void Logger::log
