@@ -40,24 +40,16 @@
 #include "DlAbortEx.h"
 #include "Peer.h"
 #include "message.h"
-#include "Socket.h"
+#include "SocketCore.h"
 #include "Logger.h"
 #include "prefs.h"
 #include "Option.h"
 #include "MSEHandshake.h"
 #include "ARC4Encryptor.h"
-#include "ARC4Decryptor.h"
 #include "RequestGroupMan.h"
 #include "BtRegistry.h"
 #include "DownloadContext.h"
-#include "PeerStorage.h"
-#include "PieceStorage.h"
-#include "BtAnnounce.h"
-#include "BtRuntime.h"
-#include "BtProgressInfoFile.h"
-#include "ServerStatMan.h"
-#include "FileAllocationEntry.h"
-#include "CheckIntegrityEntry.h"
+#include "array_fun.h"
 
 namespace aria2 {
 
@@ -72,6 +64,7 @@ ReceiverMSEHandshakeCommand::ReceiverMSEHandshakeCommand
   mseHandshake_(new MSEHandshake(cuid, s, e->getOption()))
 {
   setTimeout(e->getOption()->getAsInt(PREF_PEER_CONNECTION_TIMEOUT));
+  mseHandshake_->setWantRead(true);
 }
 
 ReceiverMSEHandshakeCommand::~ReceiverMSEHandshakeCommand()
@@ -87,102 +80,125 @@ bool ReceiverMSEHandshakeCommand::exitBeforeExecute()
 
 bool ReceiverMSEHandshakeCommand::executeInternal()
 {
-  switch(sequence_) {
-  case RECEIVER_IDENTIFY_HANDSHAKE: {
-    MSEHandshake::HANDSHAKE_TYPE type = mseHandshake_->identifyHandshakeType();
-    switch(type) {
-    case MSEHandshake::HANDSHAKE_NOT_YET:
-      break;
-    case MSEHandshake::HANDSHAKE_ENCRYPTED:
-      mseHandshake_->initEncryptionFacility(false);
-      sequence_ = RECEIVER_WAIT_KEY;
-      break;
-    case MSEHandshake::HANDSHAKE_LEGACY: {
-      if(getDownloadEngine()->getOption()->getAsBool(PREF_BT_REQUIRE_CRYPTO)) {
-        throw DL_ABORT_EX
-          ("The legacy BitTorrent handshake is not acceptable by the"
-           " preference.");
-      }
-      SharedHandle<PeerConnection> peerConnection
-        (new PeerConnection(getCuid(), getSocket()));
-      peerConnection->presetBuffer(mseHandshake_->getBuffer(),
-                                   mseHandshake_->getBufferLength());
-      Command* c = new PeerReceiveHandshakeCommand(getCuid(),
-                                                   getPeer(),
-                                                   getDownloadEngine(),
-                                                   getSocket(),
-                                                   peerConnection);
-      getDownloadEngine()->addCommand(c);
-      return true;
-    }
-    default:
-      throw DL_ABORT_EX("Not supported handshake type.");
-    }
-    break;
+  if(mseHandshake_->getWantRead()) {
+    mseHandshake_->read();
   }
-  case RECEIVER_WAIT_KEY: {
-    if(mseHandshake_->receivePublicKey()) {
-      if(mseHandshake_->sendPublicKey()) {
+  bool done = false;
+  while(!done) {
+    switch(sequence_) {
+    case RECEIVER_IDENTIFY_HANDSHAKE: {
+      MSEHandshake::HANDSHAKE_TYPE type =
+        mseHandshake_->identifyHandshakeType();
+      switch(type) {
+      case MSEHandshake::HANDSHAKE_NOT_YET:
+        done = true;
+        break;
+      case MSEHandshake::HANDSHAKE_ENCRYPTED:
+        mseHandshake_->initEncryptionFacility(false);
+        sequence_ = RECEIVER_WAIT_KEY;
+        break;
+      case MSEHandshake::HANDSHAKE_LEGACY: {
+        if(getDownloadEngine()->getOption()->getAsBool(PREF_BT_REQUIRE_CRYPTO)){
+          throw DL_ABORT_EX
+            ("The legacy BitTorrent handshake is not acceptable by the"
+             " preference.");
+        }
+        SharedHandle<PeerConnection> peerConnection
+          (new PeerConnection(getCuid(), getPeer(), getSocket()));
+        peerConnection->presetBuffer(mseHandshake_->getBuffer(),
+                                     mseHandshake_->getBufferLength());
+        Command* c = new PeerReceiveHandshakeCommand(getCuid(),
+                                                     getPeer(),
+                                                     getDownloadEngine(),
+                                                     getSocket(),
+                                                     peerConnection);
+        getDownloadEngine()->addCommand(c);
+        return true;
+      }
+      default:
+        throw DL_ABORT_EX("Not supported handshake type.");
+      }
+      break;
+    }
+    case RECEIVER_WAIT_KEY: {
+      if(mseHandshake_->receivePublicKey()) {
+        mseHandshake_->sendPublicKey();
+        sequence_ = RECEIVER_SEND_KEY_PENDING;
+      } else {
+        done = true;
+      }
+      break;
+    }
+    case RECEIVER_SEND_KEY_PENDING:
+      if(mseHandshake_->send()) {
         sequence_ = RECEIVER_FIND_HASH_MARKER;
       } else {
-        setWriteCheckSocket(getSocket());
-        sequence_ = RECEIVER_SEND_KEY_PENDING;
+        done = true;
       }
+      break;
+    case RECEIVER_FIND_HASH_MARKER: {
+      if(mseHandshake_->findReceiverHashMarker()) {
+        sequence_ = RECEIVER_RECEIVE_PAD_C_LENGTH;
+      } else {
+        done = true;
+      }
+      break;
     }
-    break;
-  }
-  case RECEIVER_SEND_KEY_PENDING:
-    if(mseHandshake_->sendPublicKey()) {
-      disableWriteCheckSocket();
-      sequence_ = RECEIVER_FIND_HASH_MARKER;
+    case RECEIVER_RECEIVE_PAD_C_LENGTH: {
+      std::vector<SharedHandle<DownloadContext> > downloadContexts;
+      getDownloadEngine()->getBtRegistry()->getAllDownloadContext
+        (std::back_inserter(downloadContexts));
+      if(mseHandshake_->receiveReceiverHashAndPadCLength(downloadContexts)) {
+        sequence_ = RECEIVER_RECEIVE_PAD_C;
+      } else {
+        done = true;
+      }
+      break;
     }
-    break;
-  case RECEIVER_FIND_HASH_MARKER: {
-    if(mseHandshake_->findReceiverHashMarker()) {
-      sequence_ = RECEIVER_RECEIVE_PAD_C_LENGTH;
+    case RECEIVER_RECEIVE_PAD_C: {
+      if(mseHandshake_->receivePad()) {
+        sequence_ = RECEIVER_RECEIVE_IA_LENGTH;
+      } else {
+        done = true;
+      }
+      break;
     }
-    break;
-  }
-  case RECEIVER_RECEIVE_PAD_C_LENGTH: {
-    std::vector<SharedHandle<DownloadContext> > downloadContexts;
-    getDownloadEngine()->getBtRegistry()->getAllDownloadContext
-      (std::back_inserter(downloadContexts));
-    if(mseHandshake_->receiveReceiverHashAndPadCLength(downloadContexts)) {
-      sequence_ = RECEIVER_RECEIVE_PAD_C;
+    case RECEIVER_RECEIVE_IA_LENGTH: {
+      if(mseHandshake_->receiveReceiverIALength()) {
+        sequence_ = RECEIVER_RECEIVE_IA;
+      } else {
+        done = true;
+      }
+      break;
     }
-    break;
-  }
-  case RECEIVER_RECEIVE_PAD_C: {
-    if(mseHandshake_->receivePad()) {
-      sequence_ = RECEIVER_RECEIVE_IA_LENGTH;
+    case RECEIVER_RECEIVE_IA: {
+      if(mseHandshake_->receiveReceiverIA()) {
+        mseHandshake_->sendReceiverStep2();
+        sequence_ = RECEIVER_SEND_STEP2_PENDING;
+      } else {
+        done = true;
+      }
+      break;
     }
-    break;
-  }
-  case RECEIVER_RECEIVE_IA_LENGTH: {
-    if(mseHandshake_->receiveReceiverIALength()) {
-      sequence_ = RECEIVER_RECEIVE_IA;
-    }
-    break;
-  }
-  case RECEIVER_RECEIVE_IA: {
-    if(mseHandshake_->receiveReceiverIA()) {
-      if(mseHandshake_->sendReceiverStep2()) {
+    case RECEIVER_SEND_STEP2_PENDING:
+      if(mseHandshake_->send()) {
         createCommand();
         return true;
       } else {
-        setWriteCheckSocket(getSocket());
-        sequence_ = RECEIVER_SEND_STEP2_PENDING;
+        done = true;
       }
+      break;
     }
-    break;
   }
-  case RECEIVER_SEND_STEP2_PENDING:
-    if(mseHandshake_->sendReceiverStep2()) {
-      disableWriteCheckSocket();
-      createCommand();
-      return true;
-    }
-    break;
+  if(mseHandshake_->getWantRead()) {
+    setReadCheckSocket(getSocket());
+  } else {
+    disableReadCheckSocket();
+  }
+  if(mseHandshake_->getWantWrite()) {
+    setWriteCheckSocket(getSocket());
+  } else {
+    disableWriteCheckSocket();
   }
   getDownloadEngine()->addCommand(this);
   return false;
@@ -191,15 +207,15 @@ bool ReceiverMSEHandshakeCommand::executeInternal()
 void ReceiverMSEHandshakeCommand::createCommand()
 {
   SharedHandle<PeerConnection> peerConnection
-    (new PeerConnection(getCuid(), getSocket()));
+    (new PeerConnection(getCuid(), getPeer(), getSocket()));
   if(mseHandshake_->getNegotiatedCryptoType() == MSEHandshake::CRYPTO_ARC4) {
     peerConnection->enableEncryption(mseHandshake_->getEncryptor(),
                                      mseHandshake_->getDecryptor());
   }
-  if(mseHandshake_->getIALength() > 0) {
-    peerConnection->presetBuffer(mseHandshake_->getIA(),
-                                 mseHandshake_->getIALength());
-  }
+  // Since initiator cannot send payload stream before reading step2
+  // from receiver, mseHandshake_->getBufferLength() should be 0.
+  peerConnection->presetBuffer(mseHandshake_->getIA(),
+                               mseHandshake_->getIALength());
   // TODO add mseHandshake_->getInfoHash() to PeerReceiveHandshakeCommand
   // as a hint. If this info hash and one in BitTorrent Handshake does not
   // match, then drop connection.
