@@ -39,13 +39,15 @@
 
 #include <string>
 #include <deque>
-#include <iosfwd>
 #include <vector>
+#include <map>
 
 #include "SharedHandle.h"
 #include "DownloadResult.h"
 #include "TransferStat.h"
 #include "RequestGroup.h"
+#include "NetStat.h"
+#include "IndexedList.h"
 
 namespace aria2 {
 
@@ -55,21 +57,32 @@ struct DownloadResult;
 class ServerStatMan;
 class ServerStat;
 class Option;
+class OutputFile;
+class UriListParser;
+class WrDiskCache;
+
+typedef IndexedList<a2_gid_t,
+                    SharedHandle<RequestGroup> > RequestGroupList;
+typedef IndexedList<a2_gid_t,
+                    SharedHandle<DownloadResult> > DownloadResultList;
 
 class RequestGroupMan {
 private:
-  std::deque<SharedHandle<RequestGroup> > requestGroups_;
-  std::deque<SharedHandle<RequestGroup> > reservedGroups_;
-  std::deque<SharedHandle<DownloadResult> > downloadResults_;
-  unsigned int maxSimultaneousDownloads_;
+  RequestGroupList requestGroups_;
+  RequestGroupList reservedGroups_;
+  DownloadResultList downloadResults_;
+
+  int maxSimultaneousDownloads_;
 
   const Option* option_;
 
   SharedHandle<ServerStatMan> serverStatMan_;
 
-  unsigned int maxOverallDownloadSpeedLimit_;
+  int maxOverallDownloadSpeedLimit_;
 
-  unsigned int maxOverallUploadSpeedLimit_;
+  int maxOverallUploadSpeedLimit_;
+
+  NetStat netStat_;
 
   // true if JSON-RPC/XML-RPC is enabled.
   bool rpc_;
@@ -78,22 +91,36 @@ private:
 
   // The number of error DownloadResult removed because of upper limit
   // of the queue
-  size_t removedErrorResult_;
+  int removedErrorResult_;
 
   // The last error of removed DownloadResult
   error_code::Value removedLastErrorResult_;
 
   size_t maxDownloadResult_;
 
-  std::string
-  formatDownloadResult(const std::string& status,
-                       const SharedHandle<DownloadResult>& downloadResult) const;
+  // UriListParser for deferred input.
+  SharedHandle<UriListParser> uriListParser_;
+
+  WrDiskCache* wrDiskCache_;
+
+  void formatDownloadResultFull
+  (OutputFile& out,
+   const char* status,
+   const SharedHandle<DownloadResult>& downloadResult) const;
+
+  std::string formatDownloadResult
+  (const char* status,
+   const SharedHandle<DownloadResult>& downloadResult) const;
 
   void configureRequestGroup
   (const SharedHandle<RequestGroup>& requestGroup) const;
+
+  void addRequestGroupIndex(const SharedHandle<RequestGroup>& group);
+  void addRequestGroupIndex
+  (const std::vector<SharedHandle<RequestGroup> >& groups);
 public:
   RequestGroupMan(const std::vector<SharedHandle<RequestGroup> >& requestGroups,
-                  unsigned int maxSimultaneousDownloads,
+                  int maxSimultaneousDownloads,
                   const Option* option);
 
   ~RequestGroupMan();
@@ -103,7 +130,7 @@ public:
   void save();
 
   void closeFile();
-  
+
   void halt();
 
   void forceHalt();
@@ -112,6 +139,9 @@ public:
 
   void fillRequestGroupFromReserver(DownloadEngine* e);
 
+  // Note that this method does not call addRequestGroupIndex(). This
+  // method should be considered as private, but exposed for unit
+  // testing purpose.
   void addRequestGroup(const SharedHandle<RequestGroup>& group);
 
   void addReservedGroup(const std::vector<SharedHandle<RequestGroup> >& groups);
@@ -124,28 +154,20 @@ public:
   void insertReservedGroup(size_t pos, const SharedHandle<RequestGroup>& group);
 
   size_t countRequestGroup() const;
-                  
-  SharedHandle<RequestGroup> getRequestGroup(size_t index) const;
-  
-  const std::deque<SharedHandle<RequestGroup> >& getRequestGroups() const
+
+  const RequestGroupList& getRequestGroups() const
   {
     return requestGroups_;
   }
 
-  SharedHandle<RequestGroup> findRequestGroup(a2_gid_t gid) const;
-
-  const std::deque<SharedHandle<RequestGroup> >& getReservedGroups() const
+  const RequestGroupList& getReservedGroups() const
   {
     return reservedGroups_;
   }
 
-  SharedHandle<RequestGroup> findReservedGroup(a2_gid_t gid) const;
-
-  enum HOW {
-    POS_SET,
-    POS_CUR,
-    POS_END
-  };
+  // Returns RequestGroup object whose gid is gid. This method returns
+  // RequestGroup either in requestGroups_ or reservedGroups_.
+  SharedHandle<RequestGroup> findGroup(a2_gid_t gid) const;
 
   // Changes the position of download denoted by gid.  If how is
   // POS_SET, it moves the download to a position relative to the
@@ -156,11 +178,12 @@ public:
   // beyond the end of the queue, it moves the download to the
   // beginning or the end of the queue respectively.  Returns the
   // destination position.
-  size_t changeReservedGroupPosition(a2_gid_t gid, int pos, HOW how);
+  size_t changeReservedGroupPosition(a2_gid_t gid, int pos,
+                                     A2_HOW how);
 
   bool removeReservedGroup(a2_gid_t gid);
 
-  void showDownloadResults(std::ostream& o) const;
+  void showDownloadResults(OutputFile& o, bool full) const;
 
   bool isSameFileBeingDownloaded(RequestGroup* requestGroup) const;
 
@@ -168,21 +191,24 @@ public:
 
   class DownloadStat {
   private:
-    size_t completed_;
-    size_t error_;
-    size_t inProgress_;
-    size_t waiting_;
+    int completed_;
+    int error_;
+    int inProgress_;
+    int removed_;
+    int waiting_;
     error_code::Value lastErrorResult_;
   public:
-    DownloadStat(size_t completed,
-                 size_t error,
-                 size_t inProgress,
-                 size_t waiting,
+    DownloadStat(int completed,
+                 int error,
+                 int inProgress,
+                 int removed,
+                 int waiting,
                  error_code::Value lastErrorResult =
                  error_code::FINISHED):
       completed_(completed),
       error_(error),
       inProgress_(inProgress),
+      removed_(removed),
       waiting_(waiting),
       lastErrorResult_(lastErrorResult) {}
 
@@ -196,7 +222,7 @@ public:
       return error_ == 0 && inProgress_ == 0 && waiting_ == 0;
     }
 
-    size_t getInProgress() const
+    int getInProgress() const
     {
       return inProgress_;
     }
@@ -204,7 +230,7 @@ public:
 
   DownloadStat getDownloadStat() const;
 
-  const std::deque<SharedHandle<DownloadResult> >& getDownloadResults() const
+  const DownloadResultList& getDownloadResults() const
   {
     return downloadResults_;
   }
@@ -241,12 +267,12 @@ public:
   // maxOverallDownloadSpeedLimit_ == 0.  Otherwise returns false.
   bool doesOverallDownloadSpeedExceed();
 
-  void setMaxOverallDownloadSpeedLimit(unsigned int speed)
+  void setMaxOverallDownloadSpeedLimit(int speed)
   {
     maxOverallDownloadSpeedLimit_ = speed;
   }
 
-  unsigned int getMaxOverallDownloadSpeedLimit() const
+  int getMaxOverallDownloadSpeedLimit() const
   {
     return maxOverallDownloadSpeedLimit_;
   }
@@ -256,17 +282,17 @@ public:
   // maxOverallUploadSpeedLimit_ == 0. Otherwise returns false.
   bool doesOverallUploadSpeedExceed();
 
-  void setMaxOverallUploadSpeedLimit(unsigned int speed)
+  void setMaxOverallUploadSpeedLimit(int speed)
   {
     maxOverallUploadSpeedLimit_ = speed;
   }
 
-  unsigned int getMaxOverallUploadSpeedLimit() const
+  int getMaxOverallUploadSpeedLimit() const
   {
     return maxOverallUploadSpeedLimit_;
   }
 
-  void setMaxSimultaneousDownloads(unsigned int max)
+  void setMaxSimultaneousDownloads(int max)
   {
     maxSimultaneousDownloads_ = max;
   }
@@ -291,9 +317,33 @@ public:
 
   // Returns currently used hosts and its use count.
   void getUsedHosts(std::vector<std::pair<size_t, std::string> >& usedHosts);
-};
 
-typedef SharedHandle<RequestGroupMan> RequestGroupManHandle;
+  const SharedHandle<ServerStatMan>& getServerStatMan() const
+  {
+    return serverStatMan_;
+  }
+
+  void setMaxDownloadResult(size_t v)
+  {
+    maxDownloadResult_ = v;
+  }
+
+  void setUriListParser(const SharedHandle<UriListParser>& uriListParser);
+
+  NetStat& getNetStat()
+  {
+    return netStat_;
+  }
+
+  WrDiskCache* getWrDiskCache() const
+  {
+    return wrDiskCache_;
+  }
+
+  // Initializes WrDiskCache according to PREF_DISK_CACHE option.  If
+  // its value is 0, cache storage will not be initialized.
+  void initWrDiskCache();
+};
 
 } // namespace aria2
 
