@@ -94,7 +94,9 @@
 #define TEXT_PROXY_METHOD                                               \
   _(" --proxy-method=METHOD        Set the method to use in proxy request.")
 #define TEXT_REFERER                                                    \
-  _(" --referer=REFERER            Set Referer. This affects all URLs.")
+  _(" --referer=REFERER            Set Referer. This affects all URLs. If \"*\" is\n" \
+    "                              given, each request URI is used as a referer.\n" \
+    "                              This may be useful when used with -P option.")
 #define TEXT_FTP_USER                                                   \
   _(" --ftp-user=USER              Set FTP user. This affects all URLs.")
 #define TEXT_FTP_PASSWD                                                 \
@@ -157,7 +159,7 @@
   _(" --allow-piece-length-change[=true|false] If false is given, aria2 aborts\n" \
     "                              download when a piece length is different from\n" \
     "                              one in a control file. If true is given, you can\n" \
-    "                              proceed but some download progress will be lost.")  
+    "                              proceed but some download progress will be lost.")
 #define TEXT_FORCE_SEQUENTIAL                                           \
   _(" -Z, --force-sequential[=true|false] Fetch URIs in the command-line sequentially\n" \
     "                              and download each URI in a separate session, like\n" \
@@ -422,7 +424,8 @@
   _(" --summary-interval=SEC       Set interval to output download progress summary.\n" \
     "                              Setting 0 suppresses the output.")
 #define TEXT_LOG_LEVEL                                          \
-  _(" --log-level=LEVEL            Set log level to output.")
+  _(" --log-level=LEVEL            Set log level to output to file specified using\n" \
+    "                             --log option.")
 #define TEXT_REMOTE_TIME                                                \
   _(" -R, --remote-time[=true|false] Retrieve timestamp of the remote file from the\n" \
     "                              remote HTTP/FTP server and if it is available,\n" \
@@ -880,3 +883,69 @@
     "                              your disk.")
 #define TEXT_ENABLE_MMAP                        \
   _(" --enable-mmap[=true|false]   Map files into memory.")
+#define TEXT_RPC_CERTIFICATE                                            \
+  _(" --rpc-certificate=FILE       Use the certificate in FILE for RPC server.\n" \
+    "                              The certificate must be in PEM format.\n" \
+    "                              Use --rpc-private-key option to specify the\n" \
+    "                              private key. Use --rpc-secure option to enable\n" \
+    "                              encryption.")
+#define TEXT_RPC_PRIVATE_KEY                                            \
+  _(" --rpc-private-key=FILE       Use the private key in FILE for RPC server.\n" \
+    "                              The private key must be decrypted and in PEM\n" \
+    "                              format. Use --rpc-secure option to enable\n" \
+    "                              encryption. See also --rpc-certificate option.")
+#define TEXT_RPC_SECURE                         \
+  _(" --rpc-secure[=true|false]    RPC transport will be encrypted by SSL/TLS.\n" \
+    "                              The RPC clients must use https scheme to access\n" \
+    "                              the server. For WebSocket client, use wss\n" \
+    "                              scheme. Use --rpc-certificate and\n" \
+    "                              --rpc-private-key options to specify the\n" \
+    "                              server certificate and private key.")
+#define TEXT_RPC_SAVE_UPLOAD_METADATA                                   \
+  _(" --rpc-save-upload-metadata[=true|false] Save the uploaded torrent or\n" \
+    "                              metalink metadata in the directory specified\n" \
+    "                              by --dir option. The filename consists of\n" \
+    "                              SHA-1 hash hex string of metadata plus\n" \
+    "                              extension. For torrent, the extension is\n" \
+    "                              '.torrent'. For metalink, it is '.meta4'.\n" \
+    "                              If false is given to this option, the\n" \
+    "                              downloads added by aria2.addTorrent or\n" \
+    "                              aria2.addMetalink will not be saved by\n" \
+    "                              --save-session option.")
+#define TEXT_FORCE_SAVE                         \
+  _(" --force-save[=true|false]    Save download with --save-session option even\n" \
+    "                              if the download is completed or removed. This\n" \
+    "                              may be useful to save BitTorrent seeding which\n" \
+    "                              is recognized as completed state.")
+#define TEXT_DISK_CACHE                         \
+  _(" --disk-cache=SIZE            Enable disk cache. If SIZE is 0, the disk cache\n" \
+    "                              is disabled. This feature caches the downloaded\n" \
+    "                              data in memory, which grows to at most SIZE\n" \
+    "                              bytes. The cache storage is created for aria2\n" \
+    "                              instance and shared by all downloads. The one\n" \
+    "                              advantage of the disk cache is reduce the disk\n" \
+    "                              I/O because the data are written in larger unit\n" \
+    "                              and it is reordered by the offset of the file.\n" \
+    "                              If hash checking is involved and the data are\n" \
+    "                              cached in memory, we don't need to read them\n" \
+    "                              from the disk.\n"                    \
+    "                              SIZE can include K or M(1K = 1024, 1M = 1024K).")
+#define TEXT_GID                                \
+  _(" --gid=GID                    Set GID manually. aria2 identifies each\n" \
+    "                              download by the ID called GID. The GID must be\n" \
+    "                              hex string of 16 characters, thus [0-9a-zA-Z]\n" \
+    "                              are allowed and leading zeros must not be\n" \
+    "                              stripped. The GID all 0 is reserved and must\n" \
+    "                              not be used. The GID must be unique, otherwise\n" \
+    "                              error is reported and the download is not added.\n" \
+    "                              This option is useful when restoring the\n" \
+    "                              sessions saved using --save-session option. If\n" \
+    "                              this option is not used, new GID is generated\n" \
+    "                              by aria2.")
+#define TEXT_CONSOLE_LOG_LEVEL                                          \
+  _(" --console-log-level=LEVEL    Set log level to output to console.")
+#define TEXT_SAVE_SESSION_INTERVAL                                      \
+  _(" --save-session-interval=SEC  Save error/unfinished downloads to a file\n" \
+    "                              specified by --save-session option every SEC\n" \
+    "                              seconds. If 0 is given, file will be saved only\n" \
+    "                              when aria2 exits.")
