@@ -1,7 +1,7 @@
 #include "DHTRoutingTableSerializer.h"
 
 #include <cstring>
-#include <sstream>
+#include <fstream>
 #include <iostream>
 
 #include <cppunit/extensions/HelperMacros.h>
@@ -20,9 +20,7 @@ class DHTRoutingTableSerializerTest:public CppUnit::TestFixture {
 
   CPPUNIT_TEST_SUITE(DHTRoutingTableSerializerTest);
   CPPUNIT_TEST(testSerialize);
-#ifdef HAVE_INET_NTOP
   CPPUNIT_TEST(testSerialize6);
-#endif // HAVE_INET_NTOP
   CPPUNIT_TEST_SUITE_END();
 private:
   char zero[256];
@@ -110,13 +108,15 @@ void DHTRoutingTableSerializerTest::testSerialize()
   }
   nodesSrc[1]->setIPAddress("non-numerical-name");
   std::vector<SharedHandle<DHTNode> > nodes(vbegin(nodesSrc), vend(nodesSrc));
-  
+
   DHTRoutingTableSerializer s(AF_INET);
   s.setLocalNode(localNode);
   s.setNodes(nodes);
 
-  std::stringstream ss;
-  s.serialize(ss);
+  std::string filename = A2_TEST_OUT_DIR"/aria2_DHTRoutingTableSerializerTest_testSerialize";
+  s.serialize(filename);
+
+  std::ifstream ss(filename.c_str(), std::ios::binary);
 
   checkToLocalnode(ss, localNode);
   size_t numNodes = 3;
@@ -224,7 +224,6 @@ void DHTRoutingTableSerializerTest::testSerialize()
   CPPUNIT_ASSERT(ss.eof());
 }
 
-#ifdef HAVE_INET_NTOP
 void DHTRoutingTableSerializerTest::testSerialize6()
 {
   SharedHandle<DHTNode> localNode(new DHTNode());
@@ -237,13 +236,15 @@ void DHTRoutingTableSerializerTest::testSerialize6()
   }
   nodesSrc[1]->setIPAddress("non-numerical-name");
   std::vector<SharedHandle<DHTNode> > nodes(vbegin(nodesSrc), vend(nodesSrc));
-  
+
   DHTRoutingTableSerializer s(AF_INET6);
   s.setLocalNode(localNode);
   s.setNodes(nodes);
 
-  std::stringstream ss;
-  s.serialize(ss);
+  std::string filename = A2_TEST_OUT_DIR"/aria2_DHTRoutingTableSerializerTest_testSerialize6";
+  s.serialize(filename);
+
+  std::ifstream ss(filename.c_str(), std::ios::binary);
 
   checkToLocalnode(ss, localNode);
   size_t numNodes = 2;
@@ -312,6 +313,5 @@ void DHTRoutingTableSerializerTest::testSerialize6()
   CPPUNIT_ASSERT_EQUAL((std::streamsize)0, ss.gcount());
   CPPUNIT_ASSERT(ss.eof());
 }
-#endif // HAVE_INET_NTOP
 
 } // namespace aria2
